#!/bin/bash

set -e
DISTRIBUTION_USER_NAME=${DISTRIBUTION_USER_NAME:-"distribution"}
DISTRIBUTION_USER_GROUP=${DISTRIBUTION_USER_GROUP:-"distribution"}
# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'

MAX_OPEN_FILES_MIN_VALUE=10000
MAX_OPEN_FILES_REQUIRED_VALUE=32000

# DISTRIBUTION_APP_SERVICES_LIST, the services names as they are in the docker-compose file, SPACE separated
DISTRIBUTION_APP_SERVICES_LIST="distributor distribution"

RECOMMENDED_MIN_RAM=8388608      # it means that for Distribution need more than 8G Total RAM => 8*1024*1024k=8388608
RECOMMENDED_MAX_USED_STORAGE=80  # it means that for Distribution need more than 20% available storage
RECOMMENDED_MIN_CPU=4
CANCEL_INSTALLER_STATE=false
VALID_USER_CHOICE=false
MAX_TRIES_INSERT_CONNECTION_STR=3

INSTALLER_INFO_FILE="/data/installer.info"
DEPLOYED="deployed"
MONGO_CONNECTION_STRING_DEFAULT='mongodb://distribution:password@mongodb:27017/distribution?authSource=distribution&authMechanism=SCRAM-SHA-1'
REDIS_CONNECTION_STRING_DEFAULT="redis://:password@redis:6379"
POSTGRES_CONNECTION_STRING_DEFAULT='jdbc:postgresql://postgres:5432/distribution?user=distribution&password=password'
POSTGRES_CONNECTION_STRING=${POSTGRES_CONNECTION_STRING:-${POSTGRES_CONNECTION_STRING_DEFAULT}}
REDIS_KEY_DISTRIBUTION="distribution.distributor.redis.connectionString"
REDIS_KEY_DISTRIBUTOR="distributor.redis.connectionString"

# Installed version
INSTALLER_DATA_FOLDER="/data"
INSTALLED_VERSION_FILE="${INSTALLER_DATA_FOLDER}/version.current"
INSTALLED_VERSION_FILE_HISTORY="${INSTALLER_DATA_FOLDER}/version.history"
INSTALLED_VERSION=''
#EULA_DOC="${INSTALLER_DATA_FOLDER}/JFrog_Distribution_EULA.info"
#EULA_DOC_SRC="${INSTALLER_HOME}/JFrog_Distribution_EULA.info"
DISTRIBUTION_GLOBAL_MOUNT_ROOT="/DISTRIBUTION_GLOBAL_MOUNT_ROOT" # This should not be used, unless special maintenance is needed

VALUES_YML="${INSTALLER_HOME}/values.yml"
DOCKER_COMPOSE_GENERATED_FILE="${INSTALLER_HOME}/docker-compose.yml"

[ -f ${INSTALLED_VERSION_FILE} ] && INSTALLED_VERSION=$(cat ${INSTALLED_VERSION_FILE})

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="${INSTALLER_HOME}/version.current"
MAINTAINER_VERSION=''
[ -f ${MAINTAINER_VERSION_FILE} ] && MAINTAINER_VERSION=$(cat ${MAINTAINER_VERSION_FILE})

#load distribution configuration
DISTRIBUTION_SERVER_PORT_DEFAULT=8080
DISTRIBUTION_SERVER_PORT=$(./getDistributionConfValue.py -f "${DISTRIBUTION_YAML_FILE}" -k "port" 2>/dev/null ) || true #echo "port cannot be loaded from DISTRIBUTION_YAML_FILE. Using default value: ${DISTRIBUTION_SERVER_PORT_DEFAULT} "
DISTRIBUTION_SERVER_PORT=${DISTRIBUTION_SERVER_PORT:-${DISTRIBUTION_SERVER_PORT_DEFAULT}}
re_is_num='^[0-9]+$'
if ! [[ ${DISTRIBUTION_SERVER_PORT} =~ $re_is_num ]] ; then
   echo "ERROR: distribution.local.config.yml: port is not a number" >&2; exit 1
fi

aparse() {
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    while [[ $# > 0 ]] ; do
      case "$1" in
        --compose-flags)
            shift
              export DOCKER_COMPOSE_FLAGS=${1}
              shift
          ;;
        --distributor-flags)
              shift
              export DOCKER_COMPOSE_DISTRIBUTOR_FLAGS=${1}
              shift
          ;;
        --distribution-flags)
              shift
              export DOCKER_COMPOSE_DISTRIBUTION_FLAGS=${1}
              shift
          ;;
          *)
              ALL_ARGS="${ALL_ARGS}${1} "
              shift
          ;;
      esac
    done
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="distribution"
fi

export mongo_connectionString="mongodb://distribution:password@mongodb:27017/distribution?authSource=distribution&authMechanism=SCRAM-SHA-1"
# Default compose env
export DISTRIBUTION_SERVER_PORT=${DISTRIBUTION_SERVER_PORT}
export DISTRIBUTION_DATA="/var/opt/jfrog/distribution"
export DISTRIBUTION_HOME="/opt/jfrog/distribution-distribution"
export DOCKER_COMPOSE_FLAGS=''
export DOCKER_COMPOSE_DISTRIBUTION_FLAGS=''
export DEFAULT_JAVA_OPTS_DISTRIBUTION="-Ddistribution.home=$DISTRIBUTION_DATA -Duser.timezone=UTC -Xms2g -Xmx4g -Dspring.profiles.active=production"
export MAIN_JAR_DISTRIBUTION="$DISTRIBUTION_HOME/lib/distribution_${DISTRIBUTION_VERSION}.jar"
export DISTRIBUTION_SERVICE_NAME=distribution
export POSTGRES_USER=${POSTGRES_USER:-"distribution"}
export POSTGRES_PASSWORD=${POSTGRES_PASSWORD:-"password"}
export POSTGRES_DB=${POSTGRES_DB:-"distribution"}
export DOCKER_COMPOSE_DISTRIBUTOR_FLAGS=''
export DISTRIBUTOR_DATA="/var/opt/jfrog/distributor"
export DISTRIBUTOR_HOME="/opt/jfrog/distribution-distributor"
export DEFAULT_JAVA_OPTS_DISTRIBUTOR="-Ddistribution.home=$DISTRIBUTOR_DATA -Duser.timezone=UTC -Xms2g -Xmx4g -Dspring.profiles.active=production"
export MAIN_JAR_DISTRIBUTOR="$DISTRIBUTOR_HOME/lib/distributor_${DISTRIBUTION_VERSION}.jar"
export DISTRIBUTOR_SERVICE_NAME=distributor

exitOnError() {
    msg=$1
    echo "ERROR: ${msg}"
    exit 1
}

usage() {
    cat ./readme.txt
    exit 1
}

version_lt() { test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1"; }

# $1 should be WARNING or ERROR
printUlimitErrorMsg() {
    local error_level=$1
    local number=$2
    MSG="
${error_level}: Max number of open files [${number}] is too low !!!

Add the following to /etc/security/limits.conf file on HOST:
* soft nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}
* hard nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}

Note: Also verify docker limits configuration if exists [usually in /etc/default/docker]

"
    case ${error_level} in
    "ERROR")
        echo -e "\033[31m${MSG}\033[0m "
    ;;
    "WARNING")
        echo -e "\033[33m${MSG}\033[0m"
    ;;
    esac

    echo
}

userChoiceVerification() {
    if [[ "$1" = [nN] ]] || [[ "$1" = [yY] ]]; then
        VALID_USER_CHOICE=true
    else
        echo "Invalid option."
    fi
}

checkPrerequisites() {
    #echo "Check dependencies:"
    echo "Verifying Distribution prerequisites ..."
    TOTAL_RAM="$(grep ^MemTotal /proc/meminfo | awk '{print $2}')"
    USED_STORAGE="$(df -h /data | tail -n +2 | tr -s ' ' | tr '%' ' ' | cut -d ' ' -f 5)"
    FREE_CPU="$(grep -c ^processor /proc/cpuinfo)"
    local msg=""
    DOCKER_SERVER_CURRENT_VER="$(docker version --format '{{.Server.Version}}')"
    if [ -z ${DOCKER_SERVER_CURRENT_VER} ] || [ "${DOCKER_SERVER_CURRENT_VER}" == "" ]; then
        msg="WARNING: Docker daemon version is unknown"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
        # Do not exit on warning
    fi

    if version_lt ${DOCKER_SERVER_CURRENT_VER} ${DOCKER_SERVER_MIN_VER}; then
        exitOnError "Distribution requires Docker daemon version ${DOCKER_SERVER_MIN_VER} or later. The current version is ${DOCKER_SERVER_CURRENT_VER}."
    fi

    # echo "Check max number of open files:"
    currentMaxOpenFilesVal=$(ulimit -n)

    if [ "${currentMaxOpenFilesVal}" != "unlimited" ] ; then
        # MAX_OPEN_FILES_REQUIRED_VALUE
        if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_REQUIRED_VALUE} ]; then
            #echo "Trying to set max open files = ${MAX_OPEN_FILES_REQUIRED_VALUE}"
            ulimit -n ${MAX_OPEN_FILES_REQUIRED_VALUE} 2>/dev/null
            if [ $? -gt 0 ]; then
                # Cannot set MAX_OPEN_FILES_REQUIRED_VALUE, will try to set MAX_OPEN_FILES_MIN_VALUE
                # MAX_OPEN_FILES_MIN_VALUE
                if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_MIN_VALUE} ]; then
                    #echo "Trying to set max open files = ${MAX_OPEN_FILES_MIN_VALUE}"
                    ulimit -n ${MAX_OPEN_FILES_MIN_VALUE} 2>/dev/null
                    if [ $? -gt 0 ]; then
                        printUlimitErrorMsg "ERROR" "${currentMaxOpenFilesVal}"
                        return 1
                    fi
                fi
                printUlimitErrorMsg "WARNING" "${currentMaxOpenFilesVal}"
                CANCEL_INSTALLER_STATE=true
                # Do not exit on warning
            fi
        fi
    fi

    #Checking RAM
    if [[ ${TOTAL_RAM} -lt ${RECOMMENDED_MIN_RAM} ]]; then
        let "TOTAL_RAM_TO_SHOW = ${TOTAL_RAM} / 1024 / 1024"
        msg="WARNING: Running with ${TOTAL_RAM_TO_SHOW}GB Total RAM"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi
    #Checking disk space
    if [[ ${USED_STORAGE} -gt ${RECOMMENDED_MAX_USED_STORAGE} ]]; then
        let "AVAILABLE_STORAGE = 100 - ${USED_STORAGE}"
        msg="WARNING: Running with $AVAILABLE_STORAGE% Free Storage"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi

    #Checking cpu
    if [ ${FREE_CPU} -lt ${RECOMMENDED_MIN_CPU} ]; then
        msg="WARNING: Running with $FREE_CPU CPU Cores"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi
    if [ "${USE_DEFAULTS}" == false ]; then
        #Asking user to proceed or not
        if [ ${CANCEL_INSTALLER_STATE} != "false" ]; then
            until [ ${VALID_USER_CHOICE} == true ]; do
                 read -p "The System resources are not aligned with Distribution minimal perquisites, Do you want to proceed with the installation? [Y/n]: " USER_CHOICE
                 userChoiceVerification ${USER_CHOICE}
            done
        fi
    else
        USER_CHOICE=y
    fi

    #In case that user typed "nN", exiting"
    if [[ "${USER_CHOICE}" =~ [nN] ]]; then
        echo "Exiting..."
        exit 1
    fi

    # dependencies check is passed
    return 0
}

setupDirsPermissions() {
    local installer_mount_root="${INSTALLER_DATA_FOLDER}"
    local distribution_mount_root_distribution="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution"
    local distribution_mount_root_distributor="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distributor"

    echo "Checking permissions on ${installer_mount_root} on host"
    chown -R ${DISTRIBUTION_USER_NAME}:${DISTRIBUTION_USER_GROUP} ${installer_mount_root} || exitOnError "Setting ownership on ${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution-installer : ${installer_mount_root} (on the host : inside the container) failed "

    # Setup ${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution permissions
    echo "Checking permissions on ${distribution_mount_root_distribution} on host"
    STAT=( $(stat -c "%U %G" ${distribution_mount_root_distribution}) )
    USER=${STAT[0]}
    GROUP=${STAT[1]}
    if [[ ${USER} != "$DISTRIBUTION_USER_NAME" ]] || [[ ${GROUP} != "$DISTRIBUTION_USER_GROUP"  ]]; then
         echo "$distribution_mount_root_distribution is owned by $USER:$GROUP. Setting to $DISTRIBUTION_USER_NAME:$DISTRIBUTION_USER_GROUP."
         echo "NOTE: The following procedures change the ownership of files and may take several minutes. Do not stop the installation/upgrade process."
         chown -R ${DISTRIBUTION_USER_NAME}:${DISTRIBUTION_USER_GROUP} ${distribution_mount_root_distribution} || exitOnError "Setting ownership on $distribution_mount_root_distribution (on the host : inside the container) failed"
    else
        echo "$distribution_mount_root_distribution is already owned by $DISTRIBUTION_USER_NAME:$DISTRIBUTION_USER_GROUP."
    fi

    # Setup ${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distributor permissions
    echo "Checking permissions on ${distribution_mount_root_distributor} on host"
    STAT=( $(stat -c "%U %G" ${distribution_mount_root_distributor}) )
    USER=${STAT[0]}
    GROUP=${STAT[1]}
    if [[ ${USER} != "$DISTRIBUTION_USER_NAME" ]] || [[ ${GROUP} != "$DISTRIBUTION_USER_GROUP"  ]]; then
         echo "$distribution_mount_root_distributor is owned by $USER:$GROUP. Setting to $DISTRIBUTION_USER_NAME:$DISTRIBUTION_USER_GROUP."
         echo "NOTE: The following procedures change the ownership of files and may take several minutes. Do not stop the installation/upgrade process."
         chown -R ${DISTRIBUTION_USER_NAME}:${DISTRIBUTION_USER_GROUP} ${distribution_mount_root_distributor} || exitOnError "Setting ownership on $distribution_mount_root_distributor (on the host: inside the container) failed"
    else
        echo "$distribution_mount_root_distributor is already owned by $DISTRIBUTION_USER_NAME:$DISTRIBUTION_USER_GROUP."
    fi
}

runDockerCommand() {
    local action=$1

    case ${action} in
    "pull")
       if  [ "$DOCKER_INSTALLER_OFFLINE" == "true" ]; then
            return 0
       fi

    ;;
    "logs")
        action="logs -f"
    ;;
    "stopApp")
        # Stop Distribution App layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${DISTRIBUTION_APP_SERVICES_LIST}
        return $?
    ;;
    "restart")
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop"
        sleep 3
        action="up -d"
    ;;
    "restartApp")
        # Restart Distribution App Layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${DISTRIBUTION_APP_SERVICES_LIST}
        sleep 3
        action="up -d"
    ;;
    "clean")
        # For DEBUG only
        docker rm -f $(docker ps -a |grep ${DISTRIBUTION_VERSION} | awk '{print $1}')
        docker rmi -f $(docker images |grep ${DISTRIBUTION_VERSION} | awk '{print $3}')
        rm -rf /home/${USER}/.jfrog/distribution || exitOnError "Removing home distribution folder failed"
        return $?
    ;;
    esac

    docker-compose -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp() {
    if [[ $1 == $2 ]]; then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++)); do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++)); do
        if [[ -z ${ver2[i]} ]]; then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]})); then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]})); then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {
    local installedVersion=${INSTALLED_VERSION}
    local maintainerVersion=${MAINTAINER_VERSION}

    if [ -z ${MAINTAINER_VERSION} ]; then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z ${INSTALLED_VERSION} ]; then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]; then
        installedVersion=($(echo ${INSTALLED_VERSION} | sed "s/^b//g"))
        maintainerVersion=($(echo ${MAINTAINER_VERSION} | sed "s/^b//g"))
    fi

    verComp ${installedVersion} ${maintainerVersion} 2>/dev/null
    if [ $? -eq 1 ]; then
        return 1
    fi

    return 0
}

isCommandAllowed() {
    local command=$1
    local installedVersion=${INSTALLED_VERSION}
    local maintainerVersion=${MAINTAINER_VERSION}

    case ${command} in
        "install")
            [ -f ${INSTALLED_VERSION_FILE} ] && exitOnError "Distribution is already installed. To upgrade, use: ./distribution upgrade"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
        ;;
        "upgrade")
            [ -f ${INSTALLED_VERSION_FILE} ] || exitOnError "Distribution is not installed. To install, use: ./distribution install"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
            isUpgradeAllowed || exitOnError "Unable to upgrade to an older version"
        ;;
        "version"|"clean")

        ;;
        "info")

        ;;
        *)
            # Is installed
            [ -f ${INSTALLED_VERSION_FILE} ] || exitOnError "Distribution is not installed. To install, use: ./distribution install"
            # Is correct version
            [ "${installedVersion}" == "${maintainerVersion}" ] || exitOnError "You are using the Distribution installer version [${maintainerVersion}] which is different from the currently installed version [${installedVersion}]. Consider upgrading your version of Distribution using: ./distribution upgrade"
        ;;
    esac

    return 0
}

setVersions() {
    [ ! -f ${MAINTAINER_VERSION_FILE} ] && return 1

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

setEULA() {
    cp ${EULA_DOC_SRC} ${EULA_DOC}
    return $?
}

# backward compatible issues
# Distribution should be stopped before this process
backwardCompatibleProcess() {
    #echo -e "\nINFO: Backward compatible process"
    LOGS_FOLDER_V1="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/log"
    LOGS_FOLDER_V2="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/logs"
    CONFIG_FOLDER="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/config"
    DISTRIBUTION_ETC_FOLDER="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/etc"
    DISTRIBUTOR_ETC_FOLDER="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distributor/etc"
    ARCHIVE_FOLDER="${INSTALLER_DATA_FOLDER}/archive"
    TIMESTAMP=$(date +"%s")

#    # Rename log folder to be logs
#    if [ -d "${LOGS_FOLDER_V1}" ] && [ ! -d "${LOGS_FOLDER_V2}" ]; then
#        mv ${LOGS_FOLDER_V1} ${LOGS_FOLDER_V2} && \
#        echo "Logs folder renamed was:[${LOGS_FOLDER_V1}] new:[${LOGS_FOLDER_V2}]"
#
#        # Archive old log4go_config.xml
#        if [ ! -d ${ARCHIVE_FOLDER} ]; then
#            mkdir -p ${ARCHIVE_FOLDER} && \
#            echo "Archive folder was created [${ARCHIVE_FOLDER}]"
#        fi
#
#        for log4goService in "distribution" "distributor"; do
#           echo "Archive ${log4goService}_log4go_config.xml to ${ARCHIVE_FOLDER}"
#           if [ -f "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" ] && [ -d "${ARCHIVE_FOLDER}" ]; then
#                mv "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" "${ARCHIVE_FOLDER}/${log4goService}_log4go_config.xml.${TIMESTAMP}"
#           fi
#        done
#    fi
}

testDockerHostConnectivity() {
    prompt_user=false

    # If DISTRIBUTION_SERVER_HOST is NOT set or set but not accessible - ask user for input
    if [ -z ${DISTRIBUTION_SERVER_HOST} ]; then
        prompt_user=true
    else
        check_host_cmd="ping -c 1 ${DISTRIBUTION_SERVER_HOST}"
        eval ${check_host_cmd} > /dev/null
        ret_code=$?
        if [ ! ${ret_code} == 0 ]; then
            prompt_user=true
        fi
    fi

    if [ "${prompt_user}" == true ]; then
        if [ ! ${USE_DEFAULTS} == true ]; then
            ret_code=1
            until [ ${ret_code} == 0 ]; do
                DISTRIBUTION_SERVER_HOST=
                read -p "Provide ip address or host FQDN of the docker host where the installation is running: " DISTRIBUTION_SERVER_HOST
                check_host_cmd="ping -c 1 ${DISTRIBUTION_SERVER_HOST}"
                eval ${check_host_cmd} > /dev/null
                ret_code=$?
                if [ ! ${ret_code} == 0 ]; then
                    echo "Can not access ${DISTRIBUTION_SERVER_HOST}!!!"
                fi
            done
        else
            echo -e "ERROR: DISTRIBUTION_SERVER_HOST is not set or can't be accessed.\nPlease run 'export DISTRIBUTION_SERVER_HOST=<server>' where server should be an ip address or host FQDN of the docker host where the installation is running, and then re-run the installation"
            exit 1
        fi
    fi

    SERVER_INTERNAL_URL="http://${DISTRIBUTION_SERVER_HOST}:8080"
}

setClusterConfig() {
    DISTRIBUTION_KEY_DIR="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/.platform"
    DISTRIBUTION_MASTER_KEY_FILE="${DISTRIBUTION_KEY_DIR}/master.key"

    if [ -f ${INSTALLER_INFO_FILE} ]; then
        source ${INSTALLER_INFO_FILE}
    else
        touch ${INSTALLER_INFO_FILE}
    fi

    if [ -z ${HA_INSTALLATION} ]; then
        if [ "${USE_DEFAULTS}" == true ] ; then
            # Force standalone installation in case of "use defaults" mode and no input from INSTALLER_INFO_FILE
            HA_INSTALLATION=false
        else
            VALID_USER_CHOICE=false
            until [ ${VALID_USER_CHOICE} == true ]; do
                read -p "Are you adding this node to an existing cluster? (not relevant for the first cluster node) [Y/n]: " is_ha_installation
                userChoiceVerification ${is_ha_installation}
            done
            if [[ "${is_ha_installation}" = [yY] ]]; then
                HA_INSTALLATION=true
            else
                HA_INSTALLATION=false
            fi
        fi
    fi
    sed -i '/.*HA_INSTALLATION.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
    echo "export HA_INSTALLATION=${HA_INSTALLATION}" >> ${INSTALLER_INFO_FILE}

    if [ -f ${INSTALLER_INFO_FILE} ]; then
        source ${INSTALLER_INFO_FILE}
    fi
    if [ ! -f ${DISTRIBUTION_MASTER_KEY_FILE} ] && [ "${HA_INSTALLATION}" == true ]; then
        until [ ! -z ${DISTRIBUTION_MASTER_KEY} ]; do
            read -p "Provide the 32 bytes master key from the first Distribution node (the key can be found in <DISTRIBUTION DATA FOLDER>/.platform/master.key): " DISTRIBUTION_MASTER_KEY
        done
        mkdir -p ${DISTRIBUTION_KEY_DIR}
        echo -n ${DISTRIBUTION_MASTER_KEY} > ${DISTRIBUTION_MASTER_KEY_FILE}
    fi
}

#This function is generating "installer.info" file
getThirdpMetadata() {
#Clean install
    if [ ! -f ${DISTRIBUTION_YAML_FILE} ]; then
        if [ "${HA_INSTALLATION}" == false ]; then
            if [ "${USE_DEFAULTS}" == true ]; then
                # Force local databases installation in case of "use defaults" mode
                INSTALL_POSTGRES=y
            else
                VALID_USER_CHOICE=false
                until [ ${VALID_USER_CHOICE} == true ]; do
                    read -p "Would you like to install PostgreSQL instance? [Y/n]: " INSTALL_POSTGRES
                    userChoiceVerification ${INSTALL_POSTGRES}
                done
            fi
        else
            INSTALL_POSTGRES=n
        fi

        if [[ "${INSTALL_POSTGRES}" = [nN] ]]; then
            read -p "Provide a PostgreSQL connection string [${POSTGRES_CONNECTION_STRING_DEFAULT}]: " postgresChoice
            POSTGRES_CONNECTION_STRING=${postgresChoice:-${POSTGRES_CONNECTION_STRING_DEFAULT}}
        else
            POSTGRES_CONNECTION_STRING=${POSTGRES_CONNECTION_STRING_DEFAULT}
        fi

        # Setting vars in installer.info
        if [ ! -f ${INSTALLER_INFO_FILE} ]; then
            touch ${INSTALLER_INFO_FILE}
        fi
        # Postgres parameters
        sed -i '/.*INSTALL_POSTGRES.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export INSTALL_POSTGRES=${INSTALL_POSTGRES}" >> ${INSTALLER_INFO_FILE}
        sed -i '/.*POSTGRES_CONNECTION_STRING.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export POSTGRES_CONNECTION_STRING=\"${POSTGRES_CONNECTION_STRING}\"" >> ${INSTALLER_INFO_FILE}
    fi
}

setEtcFolder() {
    if [ ! -d "${DISTRIBUTION_ETC_FOLDER}" ]; then
        DISTRIBUTION_ETC_FOLDER="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distribution/etc"
        mkdir -p ${DISTRIBUTION_ETC_FOLDER} || errorExit "Creating Distribution etc folder failed"
    fi
    DISTRIBUTION_YAML_FILE="$DISTRIBUTION_ETC_FOLDER/distribution.local.config.yml"

    if [ ! -d "${DISTRIBUTOR_ETC_FOLDER}" ]; then
        DISTRIBUTOR_ETC_FOLDER="${DISTRIBUTION_GLOBAL_MOUNT_ROOT}/distributor/etc"
        mkdir -p ${DISTRIBUTOR_ETC_FOLDER} || errorExit "Creating Distributor etc folder failed"
    fi
    DISTRIBUTOR_YAML_FILE="$DISTRIBUTOR_ETC_FOLDER/distributor.local.config.yml"
}

generateDistributionYamlNoMongo() {
    cat << EOF > ${DISTRIBUTION_YAML_FILE}
# Distribution configuration
## This is a YAML-formatted file

schema-version: 1
server:
  internal-url: "${SERVER_INTERNAL_URL}"

redis:
  connection-string: "${REDIS_CONNECTION_STRING_DEFAULT}"

sql:
  connection-string: "${POSTGRES_CONNECTION_STRING}"
EOF

   # Must change ownership to distribution user; otherwise, application fails
   chown -R ${DISTRIBUTION_USER}:${DISTRIBUTION_GROUP} ${DISTRIBUTION_ETC_FOLDER}
   chmod 775 ${DISTRIBUTION_ETC_FOLDER} ${DISTRIBUTION_YAML_FILE}
}

generateDistributorYaml() {
    cat << EOF > ${DISTRIBUTOR_YAML_FILE}
# Distributor configuration
# This is a YAML-formatted file

schema-version: 1
distribution:
  internal-api-url: "http://bt-distribution:8080/api/internal"

redis:
  connection-string: "${REDIS_CONNECTION_STRING_DEFAULT}"

EOF

    # Must change ownership to distribution user; otherwise, application fails
    chown -R ${DISTRIBUTION_USER}:${DISTRIBUTION_GROUP} ${DISTRIBUTOR_ETC_FOLDER}
    chmod 775 ${DISTRIBUTOR_ETC_FOLDER} ${DISTRIBUTOR_YAML_FILE}
}

# This function is generating "distribution_conf.yaml" file for each etc folder (distribution and distributor)
setYamlFiles() {
    if [ ! -f ${DISTRIBUTOR_YAML_FILE} ];then
        generateDistributorYaml
    fi
    if [ ! -f ${DISTRIBUTION_YAML_FILE} ];then
        generateDistributionYamlNoMongo
    fi
    # Removing connection string fom installer.info for security concerns
    sed -i '/.*POSTGRES_CONNECTION_STRING.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
}

#This function is generating docker-compose.yml and values.yml
generateComposeFile() {
    if [ -f ${DOCKER_COMPOSE_GENERATED_FILE} ]; then
        rm -rf ${DOCKER_COMPOSE_GENERATED_FILE}
    fi
    if [ -f ${INSTALLER_INFO_FILE} ]; then
        source ${INSTALLER_INFO_FILE}
    fi
    if [[ "${INSTALL_MONGO}" = [yY] ]]; then
        sed -i '/.*mongodb.*/d' ${VALUES_YML} || echo "There is nothing to delete, skipping ..."
        echo "  mongodb: true" >> ${VALUES_YML}
    fi

    if [[ "${INSTALL_POSTGRES}" = [yY] ]]; then
        sed -i '/.*postgres.*/d' ${VALUES_YML} || echo "There is nothing to delete, skipping ..."
        echo "  postgres: true" >> ${VALUES_YML}
    fi
    #Gdenerating docker-compose.yml
    python generate-compose-file.py
    if [ $? -gt 0 ]; then
        echo "Error while generating a customized docker-compose.yml, Exiting..."
        return 1
    fi
}

##################################
############## MAIN ##############
##################################

aparse $@

isCommandAllowed ${command}

case ${command} in
    install)
        ( testDockerHostConnectivity && \
          setEtcFolder && \
          setClusterConfig && \
          getThirdpMetadata && \
          setYamlFiles && \
          generateComposeFile && \
          runDockerCommand pull && \
          setVersions && \
#          setEULA \
          setupDirsPermissions \
        ) || exitOnError "Installation failed"
        echo
        echo -e "\033[32mINFO: Distribution $MAINTAINER_VERSION was successfully installed. Run: ./distribution start\033[0m"
        echo "To access Distribution, browse to http://<server_name>:8080"
        echo "For more information, please refer to the JFrog Distribution User Guide: https://www.jfrog.com/confluence/display/Distribution/Welcome+to+JFrog+Distribution"
        echo
        ;;
    upgrade)
        ( testDockerHostConnectivity && \
          setEtcFolder && \
          getThirdpMetadata && \
          setYamlFiles && \
          generateComposeFile && \
          runDockerCommand pull && \
          runDockerCommand stopApp && \
          backwardCompatibleProcess && \
          setVersions && \
          setupDirsPermissions \
        ) || exitOnError "Upgrade failed"
        echo
        echo "INFO: Distribution was successfully upgraded. Run: ./distribution start"
        echo ""
        ;;
    start)
        generateComposeFile && \
        runDockerCommand "up -d" && \
        setupDirsPermissions
        ;;
    ps|logs|stop|kill|restart|restartApp)
        generateComposeFile && \
        runDockerCommand ${command}
        ;;
    x-rm)
        generateComposeFile && \
        runDockerCommand rm
        ;;
    version)
        if [ -f ${INSTALLED_VERSION_FILE} ]; then
            echo "Distribution version: ${INSTALLED_VERSION}"
        else
            echo "Distribution is not installed"
        fi
        ;;
    x-compose)
         xComposeCmd="docker-compose -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
         echo "Advanced user: Going to execute '${xComposeCmd}'"
         ${xComposeCmd}
        ;;
    info)
         echo "Distribution info:"
         echo "Server port: ${DISTRIBUTION_SERVER_PORT}"
         echo "Project name: ${COMPOSE_PROJECT_NAME}"
         echo "Distribution docker repo: ${DISTRIBUTION_DOCKER_REPO}"
         echo "Distribution DB docker repo: ${DISTRIBUTIONDB_DOCKER_REPO}"
         echo "Docker mount folder: ${DISTRIBUTION_GLOBAL_MOUNT_ROOT}"
        ;;
    clean)
        runDockerCommand ${command}
    ;;
    *)
        usage
        ;;
esac
